from random import *
import pygame
from pygame.locals import *

from game import *
from util import *

MUSIC_VOLUME = 0.6

class State:
	def init(self): pass
	def paint(self,screen): pass
	def repaint(self): self._paint = 1
	def update(self,screen): pass
	def loop(self): pass
	def event(self,e): pass

class Quit(State):
	def init(self): 
		self.engine.quit = 1 #HACK: assume variable engine exists
		
class Engine:
	def fnc(self,f,v=None):
		s = self.state
		if not hasattr(s,f): return 0
		f = getattr(s,f)
		if v != None: r = f(v)
		else: r = f()
		if r != None:
			self.state = r
			self.state._paint = 1
			return 1
		return 0
		
	def run(self,s):
		self.quit = 0
		self.state = s
		
		self.init()
		
		while not self.quit:
			s = self.state
			if not hasattr(s,'_init') or s._init:
				s._init = 0
				self.state.engine = self
				if self.fnc('init'): continue
			else: 
				if self.fnc('loop'): continue
			if not hasattr(s,'_paint') or s._paint:
				s._paint = 0
				if self.fnc('paint',self.screen): continue
			else: 
				if self.fnc('update',self.screen): continue
			
			for e in pygame.event.get():
				if e.type is QUIT: 
					self.state = Quit()
					continue
				elif e.type is KEYDOWN and e.key in (K_F2,K_m,K_s):
					if self.sound:
						self.music_on^=1
						pygame.mixer.music.set_volume(MUSIC_VOLUME * self.music_on)
				elif e.type is KEYDOWN and e.key in (K_f,):
					tmp = pygame.Surface((640,480),SWSURFACE,self.screen)
					tmp.blit(self.screen,(0,0))
					mode = SWSURFACE
					self.fullscreen ^= 1
					if self.fullscreen: mode |= FULLSCREEN
					self.screen = pygame.display.set_mode((640,480),mode)
					self.screen.blit(tmp,(0,0))
					pygame.display.flip()
				if self.fnc('event',e): continue
				
			self.tick()
			
	def init(self):
		self.wait = 17
		self.nt = pygame.time.get_ticks()
		
	def tick(self):
		self.ct = pygame.time.get_ticks()
		if self.ct < self.nt:
			pygame.time.wait(self.nt-self.ct)
			self.nt+=self.wait
		else: 
			self.nt = pygame.time.get_ticks()+self.wait
			
		self.gen.work()


		
class Pause(State):
	def __init__(self,*value): 
		self.text,self.state_next = value
	def paint(self,screen): 
		mycprint(screen,self.engine.fntb,(255,255,255),self.text)
		pygame.display.flip()
	def event(self,e):
		if e.type is KEYDOWN and e.key in (K_SPACE,K_RETURN,K_ESCAPE):
			return self.state_next
			
class Prompt(State):
	def __init__(self,*value): 
		self.text,self.state_yes,self.state_no = value
	def paint(self,screen): 
		mycprint(screen,self.engine.fntb,(255,255,255),self.text)
		pygame.display.flip()
	def event(self,e):
		if e.type is KEYDOWN:
			if e.key == K_y:
				return self.state_yes
			elif e.key == K_n:
				return self.state_no

class Menu(State):
	def __init__(self,smusic = 1):
		self.smusic = smusic
		
	def init(self):
		self.pos = 0
		self.opts = [
			('Play < _L >',Play()),
			('Help',Help()),
			('Credits',Credits()),
			('Quit',Quit()),
			]
		if self.smusic and self.engine.sound:
			pygame.mixer.music.load("SJUNGLE.MOD")
			pygame.mixer.music.play(-1)
			
	def paint(self,screen):
		b = self.engine
		img = pygame.image.load("title.png")
		screen.blit(img,(0,0))
		
		c = (255,160,0)
		
 		x,y = 6,0
		myprint(screen,b.fntb,(x,y),c,"Fractal Tetris")
		y+=96
		#myprint(screen,b.fntb,(x,y),c,"")
		y+=24
		
		x+=180
		myprint(screen,b.fntm,(x,y),(255,255,255),"High: %05d"%b.high)
		y+=64
		
		c1,c2 = (128,128,128),(255,255,255)
		n = 0
		for text,state in self.opts:
			c = c1
			if n == self.pos: c = c2
			text = text.replace("_L",str(b.level))
			myprint(screen,b.fntm,(x,y),c,text)
			y += 48
			n += 1
		y+=24
		myprint(screen,b.fntm,(28,y),(255,255,255),"www.imitationpickles.org")
		pygame.display.flip()
		
	def event(self,e):
		b = self.engine
		if e.type is KEYDOWN:
			if e.key == K_UP:
				self.pos = (self.pos+len(self.opts)-1)%len(self.opts)
			elif e.key == K_DOWN:
				self.pos = (self.pos+len(self.opts)+1)%len(self.opts)
			elif e.key == K_LEFT:
				b.level = max(1,b.level-1)
			elif e.key == K_RIGHT:
				b.level = min(10,b.level+1)
			elif e.key == K_RETURN:
				return self.opts[self.pos][1]
			self.engine.sfx['tick'].play()
			self.repaint()
		
	
class Help(State):
	def paint(self,screen):
		b = self.engine
		img = pygame.image.load("help.png")
		screen.blit(img,(0,0))
		
		c = (255,160,0)
		x,y = 28,0
		myprint(screen,b.fntb,(x,y),c,"Help")
		y+=96
		
		texts = ["",
			"Up - Rotate Piece ",
			"Left/Right - Move ",
			"Down - Drop",
			"",
			"F - Fullscreen",
			"M - Music",
			]
		for text in texts:
			myprint(screen,b.fntm,(x,y),(255,255,255),text)
			y += 48
		
		pygame.display.flip()
		
	def event(self,e):
		if e.type is KEYDOWN:
			return Menu(0)

class Credits(State):
	def paint(self,screen):
		b = self.engine
		img = pygame.image.load("help.png")
		screen.blit(img,(0,0))
		
		c = (255,160,0)
		x,y = 28,0
		myprint(screen,b.fntb,(x,y),c,"Credits")
		y+=96
		
		texts = ["",
			"Game: Phil Hassey ",
			"Music: unknown / ",
			"    mortimer / heartbeat",
			"",
			"pygame / sdl / python",
			"www.imitationpickles.org",
			]
		for text in texts:
			myprint(screen,b.fntm,(x,y),(255,255,255),text)
			y += 48
		
		pygame.display.flip()
		
	def event(self,e):
		if e.type is KEYDOWN:
			return Menu(0)

						
class Play(State):
	def init(self):
		game = self.engine.game
		game.level = self.engine.level
		game.reset()
		game.shape_new()
		self.dist = 0
		self.f = 0

		game.paint(self.engine.screen)
		return Pause("Get Ready!",self)
				
	def loop(self):
		game = self.engine.game
		game.frame += 1
		
		dx,dy = 0,0
		
		pressed = pygame.key.get_pressed()
		
		if game.frame%4 == 0 and pressed[K_DOWN]:
			dy = 1
			self.dist += 1
			
		if (game.frame-self.f)%7 == 0 and pressed[K_LEFT]:
			dx = -1
		if (game.frame-self.f)%7 == 0 and pressed[K_RIGHT]:
			dx = 1
			
		#speeds = [0,30,23,18,14,11,8,6,5,4,3]
		speeds = [0,60,46,36,27,21,16,13,10,7,5]
		
		speed = speeds[min(game.level,10)]
		if self.dist==0 and game.frame%speed == 0:
			dy = 1
			
		if dx != 0:
			game.shape_move(dx,0)
		
		if dy == 1:
			if not game.shape_move(0,1):
				self.engine.sfx['tick'].play()
				game.score += self.dist
				game.update_status = 1
				self.dist = 0
				game.lines_check()
				self.engine.high = max(self.engine.high,game.score)
				if not game.shape_new():
					self.update(self.engine.screen)
					self.engine.sfx['foghorn'].play()
					return Pause("Game Over",Menu())
				#print game.level,game.lines,game.score
		
		if not pressed[K_DOWN]:
			self.dist = 0
				
	def paint(self,screen):
		game = self.engine.game
		game.paint(screen)
		
	def update(self,screen):
		game = self.engine.game
		game.update(screen)
		
	def event(self,e):
		game = self.engine.game
		if e.type is KEYDOWN:
			if e.key == K_LEFT: 
				self.f = game.frame
				game.shape_move(-1,0)
			elif e.key == K_RIGHT: 
				self.f = game.frame
				game.shape_move(1,0)
			elif e.key == K_UP: 
				game.shape_rotate()
			elif e.key in (K_RETURN,K_SPACE,K_p):
				return Pause("Pause",self)
			elif e.key in (K_ESCAPE,K_q):
				return Prompt("Quit? y/n",Menu(),self)
			#elif e.key == K_F12:
			#	game.bkgr_change()
			#	self.repaint()
			#elif e.key == K_F11:
			#	pygame.image.save(game.bkgr,"%d.tga"%game.frame)
			
		
class _sound: 
	def play(self): pass

class Loading(State):
	def init(self):
		self.img = pygame.image.load("help.png")
		self.frame = 0
		
		if self.engine.sound:
			pygame.mixer.music.load("SJUNGLE.MOD")
			pygame.mixer.music.play(-1)

	def paint(self,screen):
		mycprint(screen,self.engine.fntb,(255,255,255),"Loading...")
		x,y,w,h = self.frame+(640-480)/2,350,480,40
		screen.fill((255,255,255),(x-4,y-4,w+8,h+8))
		screen.fill((0,0,0),(x-2,y-2,w+4,h+4))
		pygame.display.flip()
	def update(self,screen):
		n = 1
		#self.engine.gen.work(n)
		x,y,w,h = self.frame+(640-480)/2,350,480,40
		screen.blit(self.img,(x,y),(x,y,n,h))
		pygame.display.update((x,y,n,h))
		if self.frame == 480:
			return Menu(0)
		self.frame += n
		
		self.engine.nt = pygame.time.get_ticks() #HACK: skip ticks
		

class Init(State):
	def init(self):
		b = self.engine
		
		pygame.font.init()
		pygame.display.init()
		pygame.display.set_icon(pygame.image.load("icon.tga"))
		pygame.display.set_caption("Fractal Tetris")
		b.screen = pygame.display.set_mode((640,480),SWSURFACE)
		b.fullscreen = 0
		
		#pygame.mouse.set_visible(0)
		b.fntb = pygame.font.Font("teenbold.ttf",96)
		b.fntm = pygame.font.Font("teenbold.ttf",48)
		b.fnts = pygame.font.Font("teenbold.ttf",16)
 		b.fntg = pygame.font.Font("teenbold.ttf",24)
		
		b.game = Game(b)
		
		
		try:
			pygame.mixer.init()
			pygame.mixer.music.set_volume(MUSIC_VOLUME)
			b.music_on = 1
			b.sound = 1
			#0/0
		except: b.sound = 0
		
		b.sfx = {}
		for name in ['foghorn','tick','blip','bang','yipee']:
			if b.sound: b.sfx[name] = pygame.mixer.Sound("%s.wav"%name)
			else: b.sfx[name] = _sound()
		#b.sfx = sfx
		
		b.level = 1
		b.high = 0
		b.gen = fractals.gen()


		return Loading()
		
#myengine = Engine()
#myengine.run(Init())
 
Engine().run(Init())
