from random import *
import pygame
from pygame.locals import *

import fractals
from util import *

TW,TH = 16,16
W,H = 12,22

class Tile:
	pass
	
class Game:
	def __init__(self,engine):
		self.engine = engine
		screen = engine.screen
		self.tiles = []
		img = pygame.image.load("tiles.tga").convert_alpha(screen)
		n = 0
		for y in range(0,img.get_height(),TH): 
			for x in range(0,img.get_width(),TW):
				t = Tile()
				t.image = img.subsurface((x,y,TW,TH))
				self.tiles.append(t)
				n += 1
		self.level = 1
		self.high = 0
		
				
	def reset(self):
		self._tlayer = [[-1 for x in xrange(0,W)] for y in xrange(0,H)]
		img = pygame.image.load("grid.tga")
		self.tlayer = [[img.get_at((x,y))[0] for x in xrange(0,W)] for y in xrange(0,H)]
		self.alayer = [[0 for x in xrange(0,W)] for y in xrange(0,H)]
		self.frame = 0
		self.score = 0
		self.lines = 0
		self.clines = 0
		self.update_status = 1
		
		self.next_shape = None
		self.bkgr_n = 0
		self.bkgr_change()
		

		
	def bkgr_change(self):
		self.bkgr = self.engine.gen.get()
		
		if self.engine.sound:
			ms = ["M_FP.XM","PASS.MOD","CHAMPOLI.XM"]
			m = ms[self.level%3]
			pygame.mixer.music.load(m)
			pygame.mixer.music.play(-1)
			

		
	def paint(self,screen):
		screen.blit(self.bkgr,(0,0))
		self._tlayer = [[-1 for x in xrange(0,W)] for y in xrange(0,H)]
		self.update_status = 1
		self.update(screen)
		pygame.display.flip()
		
		
	def update(self,screen):
		b = self.engine
		ox,oy = (640-(W*TW))/2,(480-H*TH)-12
		
		updates = []
		for y in xrange(0,H):
			for x in xrange(0,W):
				n = self.tlayer[y][x]
				_n = self._tlayer[y][x]
				if n != _n:
					t = self.tiles[n]
					screen.blit(t.image,(ox+x*TW,oy+y*TH))
					self._tlayer[y][x] = n
					updates.append(pygame.Rect(ox+x*TW,oy+y*TH,TW,TH))
		
		if self.update_status:
			
			self.update_status = 0			
 			x,y = ox+8,16
			w,h =TW*(W-1),24*4-8
			r = pygame.Rect(x,y,w,h)
			screen.fill((0,0,0),(x,y,w,h))
			screen.fill((255,255,255),(x+2,y+2,w-4,h-4))
			screen.fill((0,0,0),(x+4,y+4,w-8,h-8))
			
			updates.append(r)
			
			x+=8
			y+=4
			c = (255,255,255)
			myprint(screen,b.fntg,(x,y),c,'%06d'%self.score)
			y+=24
			myprint(screen,b.fntg,(x,y),c,'Lines: %d'%self.lines)
			y+=24
			myprint(screen,b.fntg,(x,y),c,'Level: %d'%self.level)
			y+=24
			
			x,y = r.x+64+8+24+4,r.y
			
			lx,ly,hx,hy = 4,4,0,0
			for yy in xrange(0,4):
				for xx in xrange(0,4):
					if self.next_shape[yy][xx]:
						lx = min(lx,xx)
						ly = min(ly,yy)
						hx = max(hx,xx)
						hy = max(hy,yy)
			w,h = hx-lx+1,hy-ly+1
			
			x += ((r.right-x)-w*TW)/2
			y += ((r.h)-h*TH)/2
						
			for yy in xrange(0,4):
				for xx in xrange(0,4):
					if self.next_shape[yy][xx]:
						n = self.next_t
						t = self.tiles[n]
						screen.blit(t.image,(x+(xx-lx)*TW,y+(yy-ly)*TH))
					
		pygame.display.update(updates)
				
	def lines_check(self):
		lines = []
			
		for y in xrange(1,H-1):
			line = 1
			for x in xrange(1,W-1):
				if self.tlayer[y][x] == 0: line = 0
			if line: lines.append(y)
				
		if len(lines) == 0: return

		self.engine.sfx['bang'].play()
		for i in range(0,COLORS*2):
			for y in lines:
				for x in xrange(1,W-1):
					self.tlayer[y][x] = 1+(i%(COLORS-1))
			self.update(self.engine.screen)
			self.engine.tick()
			
		sc = [0,40,100,300,1200]
		self.score += sc[len(lines)] * self.level 
		self.lines += len(lines)
		self.clines += len(lines)
		self.update_status = 1
		if self.clines >= 10: 
			self.engine.sfx['yipee'].play()
			self.level += 1
			self.clines = self.clines - 10
			self.bkgr_change()
			self.paint(self.engine.screen)
			
		for y in lines:
			for yy in xrange(y,1,-1):
				for xx in xrange(1,W-1):
					self.tlayer[yy][xx] = self.tlayer[yy-1][xx]
			for xx in xrange(1,W-1):
				self.tlayer[1][xx] = 0

	def _shape_next(self):
		self.next_shape_n = randrange(0,len(shapes))
		self.next_shape_o = 0
		self.next_shape = shapes[self.next_shape_n][self.next_shape_o]
		self.next_t = self.next_shape_n + 1 #randrange(1,COLORS)
				
	def shape_new(self):
		if self.next_shape == None:
			self._shape_next()
		self.shape_n = self.next_shape_n
		self.shape_o = self.next_shape_o
		self.shape = self.next_shape
		self.t = self.next_t
		self._shape_next()
		
		self.x,self.y = W/2-2,1
		#if self.shape_n == 1: self.y -= 1 #HACK
		r = self.shape_check(self.x,self.y)
		self._shape_draw()
		self.update_status = 1
		return r
		
	def shape_check(self,x,y):
		for yy in range(0,4):
			for xx in range(0,4):
				if self.shape[yy][xx]:
					if self.tlayer[y+yy][x+xx] != 0: return 0
		return 1
		
	def _shape_draw(self):
		x,y,t = self.x,self.y,self.t
		for yy in range(0,4):
			for xx in range(0,4):
				if self.shape[yy][xx]:
					self.tlayer[y+yy][x+xx] = t
				
	def _shape_erase(self):
		x,y,t = self.x,self.y,self.t
		for yy in range(0,4):
			for xx in range(0,4):
				if self.shape[yy][xx]:
					self.tlayer[y+yy][x+xx] = 0
		
				
	def _shape_rotate(self,r=1):
		l = len(shapes[self.shape_n])
		self.shape_o = (l+self.shape_o+r)%l
		self.shape = shapes[self.shape_n][self.shape_o]
		
	def shape_rotate(self,r=1):
		self._shape_erase()
		self._shape_rotate(r)
		
		if self.shape_check(self.x,self.y):
			self._shape_draw()
			self.engine.sfx['blip'].play()
			return 1
			
		self._shape_rotate(-r)
		self._shape_draw()

		return 0
		
	def shape_move(self,dx,dy):
		self._shape_erase()
		if self.shape_check(self.x+dx,self.y+dy):
			self.x,self.y = self.x+dx,self.y+dy
			self._shape_draw()
			return 1
		self._shape_draw()
		return 0

COLORS = 8

shapes = [
	[
	[(0,0,0,0),
	 (0,1,1,0),
	 (0,1,1,0),
	 (0,0,0,0),],
	 ],
	 
	 [
	[(0,0,0,0),
	 (1,1,1,1),
	 (0,0,0,0), #this line on GB
	 (0,0,0,0),],
	[(0,1,0,0),
	 (0,1,0,0),
	 (0,1,0,0),
	 (0,1,0,0),],
	 ],
	 
	 [
	[(0,0,0,0),
	 (0,1,1,0),
	 (1,1,0,0),
	 (0,0,0,0),],
	[(1,0,0,0),
	 (1,1,0,0),
	 (0,1,0,0),
	 (0,0,0,0),],
	 ],

 	 [
	[(0,0,0,0),
	 (1,1,0,0),
	 (0,1,1,0),
	 (0,0,0,0),],
	[(0,1,0,0),
	 (1,1,0,0),
	 (1,0,0,0),
	 (0,0,0,0),],
	 ],

  	 [
	[(0,0,0,0),
	 (1,1,1,0),
	 (1,0,0,0),
	 (0,0,0,0),],
	[(0,1,0,0),
	 (0,1,0,0),
	 (0,1,1,0),
	 (0,0,0,0),],
	[(0,0,1,0),
	 (1,1,1,0),
	 (0,0,0,0),
	 (0,0,0,0),],
	[(1,1,0,0),
	 (0,1,0,0),
	 (0,1,0,0),
	 (0,0,0,0),],
	 ],

  	 [
	[(0,0,0,0),
	 (1,1,1,0),
	 (0,0,1,0),
	 (0,0,0,0),],
	[(0,1,1,0),
	 (0,1,0,0),
	 (0,1,0,0),
	 (0,0,0,0),],
	[(1,0,0,0),
	 (1,1,1,0),
	 (0,0,0,0),
	 (0,0,0,0),],
	[(0,1,0,0),
	 (0,1,0,0),
	 (1,1,0,0),
	 (0,0,0,0),],
	 ],

  	 [
	[(0,0,0,0),
	 (1,1,1,0),
	 (0,1,0,0),
	 (0,0,0,0),],
	[(0,1,0,0),
	 (0,1,1,0),
	 (0,1,0,0),
	 (0,0,0,0),],
	[(0,1,0,0),
	 (1,1,1,0),
	 (0,0,0,0),
	 (0,0,0,0),],
	[(0,1,0,0),
	 (1,1,0,0),
	 (0,1,0,0),
	 (0,0,0,0),],
	 ],

	 	 
	 ]
	 
